package sun.awt.geom;

import java.awt.geom.Rectangle2D;
import java.util.Vector;

final class Order2 extends Curve
{
    private double x0;
    private double y0;
    private double cx0;
    private double cy0;
    private double x1;
    private double y1;
    private double xmin;
    private double xmax;
    private double xcoeff0;
    private double xcoeff1;
    private double xcoeff2;
    private double ycoeff0;
    private double ycoeff1;
    private double ycoeff2;
    
    public static void insert(Vector vector, double[] array, double n, double n2, double n3, double n4, double n5, double n6, int n7) {
        if (getHorizontalParams(n2, n4, n6, array) == 0) {
            addInstance(vector, n, n2, n3, n4, n5, n6, n7);
            return;
        }
        double n8 = array[0];
        array[0] = n;
        array[1] = n2;
        array[2] = n3;
        array[3] = n4;
        array[4] = n5;
        array[5] = n6;
        split(array, 0, n8);
        int n9 = (n7 == 1) ? 0 : 4;
        int n10 = 4 - n9;
        addInstance(vector, array[n9], array[n9 + 1], array[n9 + 2], array[n9 + 3], array[n9 + 4], array[n9 + 5], n7);
        addInstance(vector, array[n10], array[n10 + 1], array[n10 + 2], array[n10 + 3], array[n10 + 4], array[n10 + 5], n7);
        return;
    }
    
    public static void addInstance(Vector vector, double n, double n2, double n3, double n4, double n5, double n6, int n7) {
        if (n2 > n6) {
            vector.add(new Order2(n5, n6, n3, n4, n, n2, -n7));
        }
        else if (n6 > n2) {
            vector.add(new Order2(n, n2, n3, n4, n5, n6, n7));
        }
    }
    
    public static int getHorizontalParams(double n, double n2, double n3, double[] array) {
        if (n <= n2 && n2 <= n3) {
            return 0;
        }
        n -= n2;
        n3 -= n2;
        double n4 = n + n3;
        if (n4 == 0.0) {
            return 0;
        }
        double n5 = n / n4;
        if (n5 <= 0.0 || n5 >= 1.0) {
            return 0;
        }
        array[0] = n5;
        return 1;
    }
    
    public static void split(double[] array, int n, double n2) {
        double n3 = array[n + 8] = array[n + 4];
        double n4 = array[n + 9] = array[n + 5];
        double n5 = array[n + 2];
        double n6 = array[n + 3];
        double n7 = n5 + (n3 - n5) * n2;
        double n8 = n6 + (n4 - n6) * n2;
        double n9 = array[n + 0];
        double n10 = array[n + 1];
        double n11 = n9 + (n5 - n9) * n2;
        double n12 = n10 + (n6 - n10) * n2;
        double n13 = n11 + (n7 - n11) * n2;
        double n14 = n12 + (n8 - n12) * n2;
        array[n + 2] = n11;
        array[n + 3] = n12;
        array[n + 4] = n13;
        array[n + 5] = n14;
        array[n + 6] = n7;
        array[n + 7] = n8;
    }
    
    public Order2(double n, double n2, double b, double cy0, double b2, double y1, int n3) {
        super(n3);
        if (cy0 < n2) {
            cy0 = n2;
        }
        else if (cy0 > y1) {
            cy0 = y1;
        }
        this.x0 = n;
        this.y0 = n2;
        this.cx0 = b;
        this.cy0 = cy0;
        this.x1 = b2;
        this.y1 = y1;
        this.xmin = Math.min(Math.min(n, b2), b);
        this.xmax = Math.max(Math.max(n, b2), b);
        this.xcoeff0 = n;
        this.xcoeff1 = b + b - n - n;
        this.xcoeff2 = n - b - b + b2;
        this.ycoeff0 = n2;
        this.ycoeff1 = cy0 + cy0 - n2 - n2;
        this.ycoeff2 = n2 - cy0 - cy0 + y1;
    }
    
    @Override
    public int getOrder() {
        return 2;
    }
    
    @Override
    public double getXTop() {
        return this.x0;
    }
    
    @Override
    public double getYTop() {
        return this.y0;
    }
    
    @Override
    public double getXBot() {
        return this.x1;
    }
    
    @Override
    public double getYBot() {
        return this.y1;
    }
    
    @Override
    public double getXMin() {
        return this.xmin;
    }
    
    @Override
    public double getXMax() {
        return this.xmax;
    }
    
    @Override
    public double getX0() {
        return (this.direction == 1) ? this.x0 : this.x1;
    }
    
    @Override
    public double getY0() {
        return (this.direction == 1) ? this.y0 : this.y1;
    }
    
    public double getCX0() {
        return this.cx0;
    }
    
    public double getCY0() {
        return this.cy0;
    }
    
    @Override
    public double getX1() {
        return (this.direction == -1) ? this.x0 : this.x1;
    }
    
    @Override
    public double getY1() {
        return (this.direction == -1) ? this.y0 : this.y1;
    }
    
    @Override
    public double XforY(double n) {
        if (n <= this.y0) {
            return this.x0;
        }
        if (n >= this.y1) {
            return this.x1;
        }
        return this.XforT(this.TforY(n));
    }
    
    @Override
    public double TforY(double n) {
        if (n <= this.y0) {
            return 0.0;
        }
        if (n >= this.y1) {
            return 1.0;
        }
        return TforY(n, this.ycoeff0, this.ycoeff1, this.ycoeff2);
    }
    
    public static double TforY(double n, double n2, double n3, double n4) {
        n2 -= n;
        if (n4 == 0.0) {
            double n8 = -n2 / n3;
            if (n8 >= 0.0 && n8 <= 1.0) {
                return n8;
            }
        }
        else {
            double a = n3 * n3 - 4.0 * n4 * n2;
            if (a >= 0.0) {
                double sqrt = Math.sqrt(a);
                if (n3 < 0.0) {
                    sqrt = -sqrt;
                }
                double n5 = (n3 + sqrt) / -2.0;
                double n6 = n5 / n4;
                if (n6 >= 0.0 && n6 <= 1.0) {
                    return n6;
                }
                if (n5 != 0.0) {
                    double n7 = n2 / n5;
                    if (n7 >= 0.0 && n7 <= 1.0) {
                        return n7;
                    }
                }
            }
        }
        return (0.0 < (n2 + (n2 + n3 + n4)) / 2.0) ? 0.0 : 1.0;
    }
    
    @Override
    public double XforT(double n) {
        return (this.xcoeff2 * n + this.xcoeff1) * n + this.xcoeff0;
    }
    
    @Override
    public double YforT(double n) {
        return (this.ycoeff2 * n + this.ycoeff1) * n + this.ycoeff0;
    }
    
    @Override
    public double dXforT(double n, int n2) {
        switch (n2) {
            case 0: {
                return (this.xcoeff2 * n + this.xcoeff1) * n + this.xcoeff0;
            }
            case 1: {
                return 2.0 * this.xcoeff2 * n + this.xcoeff1;
            }
            case 2: {
                return 2.0 * this.xcoeff2;
            }
            default: {
                return 0.0;
            }
        }
    }
    
    @Override
    public double dYforT(double n, int n2) {
        switch (n2) {
            case 0: {
                return (this.ycoeff2 * n + this.ycoeff1) * n + this.ycoeff0;
            }
            case 1: {
                return 2.0 * this.ycoeff2 * n + this.ycoeff1;
            }
            case 2: {
                return 2.0 * this.ycoeff2;
            }
            default: {
                return 0.0;
            }
        }
    }
    
    @Override
    public double nextVertical(double n, double n2) {
        double n3 = -this.xcoeff1 / (2.0 * this.xcoeff2);
        if (n3 > n && n3 < n2) {
            return n3;
        }
        return n2;
    }
    
    @Override
    public void enlarge(Rectangle2D rectangle2D) {
        rectangle2D.add(this.x0, this.y0);
        double n = -this.xcoeff1 / (2.0 * this.xcoeff2);
        if (n > 0.0 && n < 1.0) {
            rectangle2D.add(this.XforT(n), this.YforT(n));
        }
        rectangle2D.add(this.x1, this.y1);
    }
    
    @Override
    public Curve getSubCurve(double n, double n2, int n3) {
        double tforY;
        if (n <= this.y0) {
            if (n2 >= this.y1) {
                return this.getWithDirection(n3);
            }
            tforY = 0.0;
        }
        else {
            tforY = TforY(n, this.ycoeff0, this.ycoeff1, this.ycoeff2);
        }
        double tforY2;
        if (n2 >= this.y1) {
            tforY2 = 1.0;
        }
        else {
            tforY2 = TforY(n2, this.ycoeff0, this.ycoeff1, this.ycoeff2);
        }
        double[] array = { this.x0, this.y0, this.cx0, this.cy0, this.x1, this.y1, 0.0, 0.0, 0.0, 0.0 };
        if (tforY2 < 1.0) {
            split(array, 0, tforY2);
        }
        int n4;
        if (tforY <= 0.0) {
            n4 = 0;
        }
        else {
            split(array, 0, tforY / tforY2);
            n4 = 4;
        }
        return new Order2(array[n4 + 0], n, array[n4 + 2], array[n4 + 3], array[n4 + 4], n2, n3);
    }
    
    @Override
    public Curve getReversedCurve() {
        return new Order2(this.x0, this.y0, this.cx0, this.cy0, this.x1, this.y1, -this.direction);
    }
    
    @Override
    public int getSegment(double[] array) {
        array[0] = this.cx0;
        array[1] = this.cy0;
        if (this.direction == 1) {
            array[2] = this.x1;
            array[3] = this.y1;
        }
        else {
            array[2] = this.x0;
            array[3] = this.y0;
        }
        return 2;
    }
    
    @Override
    public String controlPointString() {
        return "(" + Curve.round(this.cx0) + ", " + Curve.round(this.cy0) + "), ";
    }
}
